package cn.kdan.compdf.config;

import cn.kdan.compdf.enums.RabbitMQEnum;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.TopicExchange;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * RabbitMQ配置
 * @author Bob 2022-11-01
 **/
@Configuration
public class RabbitConfig {

    /**
     *  topics 交换机
     */
    @Bean
    public TopicExchange getExchange(){
        // 参数1：  交换机名字
        // 参数2：  消息是否持久
        // 参数3：  是否自动删除
        // 参数4：  其他配置
        return new TopicExchange(RabbitMQEnum.BACKGROUND_EXCHANGE,true,false,null);
    }


    /**
     * API更新队列
     */
    @Bean
    public Queue getAPIAddQueue(){
        // 参数1 队列的名字
        // 参数2：  消息是否持久
        // 参数3： 这个队列是否让消费者多占
        // 参数4： 这个队列是否自动删除
        // 参数5：  这个队列的其他参数
        return new Queue(RabbitMQEnum.BACKGROUND_API_ADD_QUEUE,true,false,false,null);
    }
    /**
     * API删除队列
     */
    @Bean
    public Queue getAPIDelQueue(){

        return new Queue(RabbitMQEnum.BACKGROUND_API_DEL_QUEUE,true,false,false,null);
    }
    /**
     * webhooks更新队列
     */
    @Bean
    public Queue getWebhooksAddQueue(){
        return new Queue(RabbitMQEnum.BACKGROUND_WEBHOOKS_ADD_QUEUE,true,false,false,null);
    }
    /**
     * webhooks删除队列
     */
    @Bean
    public Queue getWebhooksDelQueue(){
        return new Queue(RabbitMQEnum.BACKGROUND_WEBHOOKS_DEL_QUEUE,true,false,false,null);
    }
    /**
     * webhooks删除队列
     */
    @Bean
    public Queue getWebhooksUpdQueue(){
        return new Queue(RabbitMQEnum.BACKGROUND_WEBHOOKS_UPD_QUEUE,true,false,false,null);
    }


    /**
     *  交换机绑定队列
     */
    @Bean
    public Binding getAPIAddBinding(){
        return BindingBuilder.bind(getAPIAddQueue()).to(getExchange()).with(RabbitMQEnum.BACKGROUND_API_ADD_ROUTING_KEY);
    }
    @Bean
    public Binding getAPIDelBinding(){
        return BindingBuilder.bind(getAPIDelQueue()).to(getExchange()).with(RabbitMQEnum.BACKGROUND_API_DEL_ROUTING_KEY);
    }
    @Bean
    public Binding getWebhooksAddBinding(){
        return BindingBuilder.bind(getWebhooksAddQueue()).to(getExchange()).with(RabbitMQEnum.BACKGROUND_WEBHOOKS_ADD_ROUTING_KEY);
    }
    @Bean
    public Binding getWebhooksDelBinding(){
        return BindingBuilder.bind(getWebhooksDelQueue()).to(getExchange()).with(RabbitMQEnum.BACKGROUND_WEBHOOKS_DEL_ROUTING_KEY);
    }
    @Bean
    public Binding getWebhooksUpdBinding(){
        return BindingBuilder.bind(getWebhooksUpdQueue()).to(getExchange()).with(RabbitMQEnum.BACKGROUND_WEBHOOKS_UPD_ROUTING_KEY);
    }

}
